//+------------------------------------------------------------------+
//|                                                 EventMethods.mqh |
//|                                      Copyright 2017, nicholishen |
//|                         https://www.forexfactory.com/nicholishen |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, nicholishen"
#property link      "https://www.forexfactory.com/nicholishen"
#property version   "1.00"
#property strict
#include "paneldialog.mqh"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

bool CPanelDialog::OnResize(void)
{
//--- calling the parent class method
   if(!CAppDialog::OnResize()) 
      return(false);
//--- changing the color panel width
//--- success
return(true);
}

void  CPanelDialog::MarketOrder(int type,double price,double lots,double stop,double take,string comment="QuickTrader")
{
   uint st = GetTickCount();
   if(type>1)
      return;
   lots = MathAbs(lots);
   lots = NormalizeDouble(lots,2);
   int ticket = OrderSend(_Symbol,type,lots,price,10,stop,take,comment,MAGIC);
   if(ticket<0)
   {
      m_label_result.Text("Error: "+ErrorDescription(GetLastError()));
      Print(__FUNCTION__," Error: "+ErrorDescription(GetLastError()));
      return;
   }
   int slip = 0;
   string str_price = "";
   //int slip = 3;
   //string str_price = string(Bid);
   if(OrderSelect(ticket,SELECT_BY_TICKET))
   {
      slip = type==OP_BUY ? int((OrderOpenPrice()-price)/_Point) : int((price-OrderOpenPrice())/_Point);
      str_price = string(OrderOpenPrice());
   }
   string slippage = slip != 0 ? "(slip= "+string(slip)+")" : "";
   string op = type==OP_BUY?"BUY@":"SELL@";
   m_label_result.Text(op+" "+str_price+" | "+string(GetTickCount()-st)+" ms "+slippage);
   UpdateNetPosition();

}

void CPanelDialog::OnEditLots(void)
{
   double lots = double(m_edit_lots.Text());
   SetLots(lots);
}

void CPanelDialog::OnClickReconcile(void)
{
   for(int i=OrdersTotal()-1;i>0&&!IsStopped();)
   {
      if(OrderSelect(i,SELECT_BY_POS)
         && OrderSymbol()==_Symbol 
         && OrderType()<2 
         && (!m_locked_magic || OrderMagicNumber() == MAGIC)
         )
      {
         int type = OrderType();
         int ticket = OrderTicket();
         for(int j=i-1;OrderSelect(j,SELECT_BY_POS);j--)
         {
            if(OrderSymbol()==_Symbol 
               && OrderType()<2 
               && OrderType()!=type 
               && (!m_locked_magic || OrderMagicNumber() == MAGIC)
               )
            {
               if(!OrderCloseBy(OrderTicket(),ticket))
                  Print("OrderCLoseByError: ",GetLastError());
               i=OrdersTotal();
               break; 
            }
         }
         i--;
      }
      else
         i--;
   }   
}

void CPanelDialog::OnClickLotsUp(void)
{
   double step = SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_STEP);
   SetLots(m_lots+step);
}

void CPanelDialog::OnClickLotsDn(void)
{
   double step = SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_STEP);
   SetLots(m_lots-step);
}

void  CPanelDialog::SetLots(double new_lots=0.0)
{
   double min  = SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MIN);
   double max  = SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MAX);
   if(new_lots==0.0)
   {
      if(GlobalVariableCheck(m_global_lots_var))
         m_lots = GlobalVariableGet(m_global_lots_var);
      else
      {
         m_lots = min;
         GlobalVariableSet(m_global_lots_var,m_lots);
      } 
   }
   else
   {
      m_lots = new_lots < min ? min : new_lots > max ? max : new_lots;
      m_lots = NormalizeDouble(m_lots,2);     
      GlobalVariableSet(m_global_lots_var,m_lots);
   }
   ObjectSetString(m_chart_id, m_name+"_lots_edit",OBJPROP_TEXT,DoubleToString(m_lots,2));
}

void CPanelDialog::OnEditStop(void)
{
   int min_stop = (int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_STOPS_LEVEL);
   int inp = (int)m_edit_stop.Text();
   inp = inp >= min_stop ? inp : min_stop;
   m_stop = inp;
   ObjectSetString(m_chart_id, m_name+"_spin_stop_",OBJPROP_TEXT,string(m_stop));
   //Alert("Edit stop = "+string(inp)+" minstop = "+string(min_stop));
}

void CPanelDialog::OnEditTake(void)
{
   int min_stop = (int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_STOPS_LEVEL);
   int inp = (int)m_edit_take.Text();
   inp = inp >= min_stop ? inp : min_stop;
   m_take = inp;
   ObjectSetString(m_chart_id, m_name+"_spin_take_",OBJPROP_TEXT,string(m_take));
}

void CPanelDialog::OnClickButtonClose(void)
{ 
   ExpertRemove();
}

void CPanelDialog::OnClickSell(void)
{
   RefreshRates();
   double price = Bid;
   double stop = m_stop > 0 && !AutomaticHedging ? NormalizeDouble(price + m_stop * _Point*10,Digits) : 0.0;
   double take = m_take > 0 && !AutomaticHedging ? NormalizeDouble(price - m_take * _Point*10,Digits) : 0.0;
   MarketOrder(OP_SELL,price,m_lots,stop,take);
}

void CPanelDialog::OnClickBuy()
{
   RefreshRates();
   double price = Ask;
   double stop = m_stop > 0 && !AutomaticHedging ? NormalizeDouble(price - m_stop * _Point *10,Digits) : 0.0;
   double take = m_take > 0 && !AutomaticHedging ? NormalizeDouble(price + m_take * _Point *10,Digits) : 0.0;
   MarketOrder(OP_BUY,price,m_lots,stop,take);
}

void CPanelDialog::OnClickCloseAll()
{
       for(int i=OrdersTotal()-1;i>=0;i--) 
         if(OrderSelect(i,SELECT_BY_POS)&& OrderSymbol()==_Symbol && OrderType()<2) 
           OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),1000);

}

void CPanelDialog::OnClickBE()
{
       BREAK_EVEN=!BREAK_EVEN;
       if (BREAK_EVEN) ObjectSetInteger(m_chart_id,m_name+"_be_button",OBJPROP_BGCOLOR,clBREAK_EVEN);
       else            ObjectSetInteger(m_chart_id,m_name+"_be_button",OBJPROP_BGCOLOR,CONTROLS_BUTTON_COLOR_BG);
       Print("BE=",BREAK_EVEN);
}


void CPanelDialog::OnCheckMagic()
{
   m_locked_magic = m_check_magic.Checked();
   //string mssg = m_locked_magic ? "Only matching magic orders": "All orders this symbol";
   //m_check_magic.Text(mssg);
}

void CPanelDialog::OnClickClose(void)
{
   double net_position = NetPosition();
   int ticket = -1;
   RefreshRates();
   if(net_position > 0.0)
      MarketOrder(OP_SELL,Bid,net_position,0,0,"HEDGE-CLOSE-ORDER");
   else
   if(net_position < 0.0)
      MarketOrder(OP_BUY,Ask,net_position,0,0,"HEDGE-CLOSE-ORDER");
   UpdateNetPosition();
}

void CPanelDialog::OnClickReverse(void)
{
   double net_position = NetPosition();
   int ticket = -1;
   RefreshRates();
   if(net_position > 0.0)
      MarketOrder(OP_SELL,Bid,net_position*2,0,0,"REVERSE-ORDER");
   else
   if(net_position < 0.0)
      MarketOrder(OP_BUY,Ask,net_position*2,0,0,"REVERSE-ORDER");
   UpdateNetPosition();
}

void CPanelDialog::ReconcileHedgeOrders()
{
   for(int i=OrdersTotal()-1;i>0;i--){
      if(OrderSelect(i,SELECT_BY_POS) && OrderSymbol()==_Symbol && OrderType()<2 && (!m_locked_magic || OrderMagicNumber() == MAGIC)) {
         int type = OrderType();
         int ticket = OrderTicket();
         for(int j=i-1;OrderSelect(j,SELECT_BY_POS);j--)
         {
            if(OrderSymbol()==_Symbol && OrderType()<2  && OrderType()!=type && (!m_locked_magic || OrderMagicNumber() == MAGIC)) {
               if(!OrderCloseBy(OrderTicket(),ticket)) Print("OrderCLoseByError: ",GetLastError());
               i=OrdersTotal();
               break; 
            }
         }
      }
   }   
}

double CPanelDialog::NetPosition()
{
   double net_position = 0.0;
   for(int i=OrdersTotal()-1;i>=0;i--)
   {
      if(OrderSelect(i,SELECT_BY_POS) 
         && OrderSymbol() == Symbol() 
         && OrderType() < 2 
         && (!m_locked_magic || OrderMagicNumber() == MAGIC)
         )
      {
         if(OrderType()==OP_BUY)
            net_position+=OrderLots();
         else
         if(OrderType()==OP_SELL)
            net_position-=OrderLots();
      }
   }
   return NormalizeDouble(net_position,2);
}

void CPanelDialog::UpdateNetPosition(void)
{
   double net_position = NetPosition();
   m_net_position.Text("Net Position = "+DoubleToStr(net_position,2));
}