
#property strict
#include <stdlib.mqh>
#include "Controls\\Dialog.mqh"
#include "Controls\\Panel.mqh"
#include "Controls\\Edit.mqh"
#include "Controls\\Label.mqh"
#include "Controls\\SpinEdit.mqh"
#include "Controls\\Button.mqh"
#include "Controls\\CheckBox.mqh"
#resource "res\\up_black20.bmp"
#resource "res\\down_black20.bmp"
#resource "res\\blank.bmp"
//+------------------------------------------------------------------+
//| defines                                                          |
//+------------------------------------------------------------------+
//--- indents and spacing
#define INDENT_LEFT                         (8)      // left indent (including the border width)
#define INDENT_TOP                          (8)      // top indent (including the border width)
#define INDENT_RIGHT                        (8)      // right indent (including the border width)
#define INDENT_BOTTOM                       (8)      // bottom indent (including the border width)
#define CONTROLS_GAP_X                      (8)      // spacing along the X-axis
#define CONTROLS_GAP_Y                      (8)      // spacing along the Y-axis
//--- for labels
#define LABEL_WIDTH                         (50)      // size along the X-axis
//--- for edits
#define EDIT_WIDTH                          (100)      // size along the X-axis
#define EDIT_HEIGHT                         (20)      // size along the Y-axis
//--- for base colors (RGB)
#define BASE_COLOR_MIN                      (0)       // minimum value of the color component
#define BASE_COLOR_MAX                      (255)     // maximum value of the color component
#define MAGIC                                676677

class CCheckBox2 : public CCheckBox
{
public:
   virtual bool Create(const long chart,const string name,const int subwin,const int x1,const int y1,const int x2,const int y2)
   {
      if(!CCheckBox::Create(chart,name,subwin,x1,y1,x2,y2))
         return false; 
      m_label.Font("Consolas");
      m_label.FontSize(8); 
      return true;   
   }
};
//+------------------------------------------------------------------+
//| CPanelDialog class                                               |
//| Function: main application dialog                                |
//+------------------------------------------------------------------+
class CPanelDialog : public CAppDialog
{
private:
   //--- additional controls
   CCheckBox2        m_check_magic;
   //CCheckBox         m_check_hedging;
   CButton           m_button_buy,m_button_sell,m_button_closeall,m_button_reverse,m_button_close,m_button_reconcile,m_button_be;
   CLabel            m_label_lots;
   CLabel            m_label_sl;
   CLabel            m_label_tp;
   CLabel            m_label_result; 
   CLabel            m_net_position;
   CEdit             m_edit_lots;
   CEdit             m_edit_take;
   CBmpButton        m_button_uplots;
   CBmpButton        m_button_dnlots;
   
   int               m_stop;
   int               m_take;
   string            m_global_lots_var;
   string            m_global_take_var;
   string            m_global_stop_var;
public:
   CEdit             m_edit_stop; 
   double            m_lots;
   bool              m_locked_magic;
                     CPanelDialog(void);
                    ~CPanelDialog(void);
   //--- creation
   virtual bool      Create(const long chart,const string name,const int subwin,const int x1,const int y1,const int x2,const int y2);
   //--- chart event handler
   virtual bool      OnEvent(const int id,const long &lparam,const double &dparam,const string &sparam);
   //--- properties
   virtual bool      OnInit() { UpdateNetPosition(); return true;}
   virtual void      OnTick() { UpdateNetPosition(); }
   void              OnClickClose();
   void              SetLots(double new_lots=0.0);
   void              ReconcileHedgeOrders();

protected:
   //--- internal event handlers
   virtual bool      OnResize(void);
   bool              CreateTradeButtons();
   bool              CreateEditLots();
   bool              CreateLabels();
   bool              CreateEditTargets();
   bool              CreateCheckBoxes();
   bool              CreateReconcileButton();

   void              OnClickBuy();
   void              OnClickSell();
   void              OnClickBE();
   void              OnClickCloseAll();
   void              OnClickReverse();
   void              OnClickReconcile();
   void              OnEditLots();
   void              OnClickLotsUp();
   void              OnClickLotsDn();
   void              OnEditStop();
   void              OnEditTake();
   void              OnCheckMagic();
   
   double            NetPosition();
   void              UpdateNetPosition();
   void              MarketOrder(int type,double price,double lots,double stop,double take,string comment);
   virtual void      OnClickButtonClose(void);
   //--- event handlers for additional controls

};
//+------------------------------------------------------------------+
//| Event handling                                                   |
//+------------------------------------------------------------------+
EVENT_MAP_BEGIN(CPanelDialog)
   ON_EVENT(ON_CLICK,m_button_buy,OnClickBuy)
   ON_EVENT(ON_CLICK,m_button_sell,OnClickSell)
   ON_EVENT(ON_CLICK,m_button_closeall,OnClickCloseAll)
   ON_EVENT(ON_CLICK,m_button_reverse,OnClickReverse)
   ON_EVENT(ON_CLICK,m_button_close,OnClickClose)
   ON_EVENT(ON_CLICK,m_button_be,OnClickBE)
   ON_EVENT(ON_CLICK,m_button_reconcile,OnClickReconcile)
   ON_EVENT(ON_END_EDIT,m_edit_lots,OnEditLots)
   ON_EVENT(ON_END_EDIT,m_edit_stop,OnEditStop)
   ON_EVENT(ON_END_EDIT,m_edit_take,OnEditTake)
   ON_EVENT(ON_CLICK,m_button_uplots,OnClickLotsUp)
   ON_EVENT(ON_CLICK,m_button_dnlots,OnClickLotsDn)
   ON_EVENT(ON_CHANGE,m_check_magic,OnCheckMagic)
   
EVENT_MAP_END(CAppDialog)
//+------------------------------------------------------------------+
//| Constructor                                                      |
//+------------------------------------------------------------------+
CPanelDialog::CPanelDialog(void): m_global_lots_var("__panel_lots_var_"+_Symbol),
                                  m_global_take_var("__panel_take_var_"+_Symbol),
                                  m_global_stop_var("__panel_stop_var_"+_Symbol)
{
}
//+------------------------------------------------------------------+
//| Destructor                                                       |
//+------------------------------------------------------------------+
CPanelDialog::~CPanelDialog(void)
{
}
//+------------------------------------------------------------------+
//| Creation                                                         |
//+------------------------------------------------------------------+
#include "CreateElements.mqh"
#include "EventMethods.mqh"

//+------------------------------------------------------------------+
//| Resize handler                                                   |
//+------------------------------------------------------------------+
